# Contract for the Mon::Future monad, for use with the Contracts gem
# For example:
# <tt>Contract C::Future[Num] => C::Future[Num]
# def futureSquare(l)
#   l.bind { |i| i * i }
# end</tt>
# will work only for (eg) futureSquare(Mon::Future.eventually { wait_for_number_from_thread }).
# If passed anything (or returning anything) other than a Mon::Future, a
# ContractViolation will be thrown. <b>Note:</b> since Ruby is dynamically typed,
# we can't ensure that the eventual value will be of the correct type. So:
# futureSquare(Mon::Future.eventually { wait_for_string_from_thread }) is likely
# to succeed (if the thread is still in-flight), and fail when the thread returns.
# Completed Futures don't have this issue.

module Mon
  module Contract
    require_relative 'monad_contract'
    
    class Future < MonadContract
      def valid?(val)
        # Should be a Future, and if finalized it should satisfy
        # the provided contract. Note: We can't know if an inflight
        # contract will satisfy type or not.
        val.is_a?(Mon::M::Future) and (val.is_a?(Mon::M::FutureComplete).implies(valid_nested_contract?(val._)))
      end
    end
  end
end
