# Contract for the Mon::Lazy monad, for use with the Contracts gem
# For example:
# <tt>Contract C::Lazy[Num] => C::Lazy[Num]
# def lazySquare(l)
#   l.bind { |i| i * i }
# end</tt>
# will work only for (eg) lazySquare(Mon::Lazy.eventually { perform_painful_op_only_if_necessary() }).
# If passed anything (or returning anything) other than a Mon::Lazy, a
# ContractViolation will be thrown. <b>Note:</b> since Ruby is dynamically typed,
# we can't ensure that the eventual value will be of the correct type. So:
# lazySquare(Mon::Lazy.eventually { op_returning_string }) is likely
# to succeed, and fail when the operation is actually executed.
# Finalized Lazy monads don't have this issue.

module Mon
  module Contract
    require_relative 'monad_contract'
    
    class Lazy < MonadContract
      def valid?(val)
        # Should be a Lazy (i.e. pending or final), and if it's
        # final, the value should be valid re: the provided contract
        val.is_a?(Mon::M::Lazy) and (val.is_a?(Mon::M::Final).implies(valid_nested_contract?(val._)))
      end
    end
  end
end
