
module Mon
  module Contract
    require_relative '../mon.rb'
    require 'contracts'

    class MonadContract < Contracts::CallableClass
      require_relative 'contract_helpers'

      def initialize(*vals)
        if vals.length != 1
          throw ArgumentError.new("Incorrect usage of #{ self.class.name } contract, should be #{ self.class.name }[<contract>]")
        end
        @nested_contract = vals[0]
      end

      def nested_contract
        @nested_contract
      end

      def valid?(val)
        throw RuntimeError.new("MonadContract is abstract, #valid? must be overridden!")
      end

      def valid_nested_contract?(val)
        nested_validator = Object::Contract::make_validator(@nested_contract)
        nested_validator.call(val)
      end

      def to_s
        "#{ self.class.name }[#{@nested_contract.to_s}]"
      end
    end

    class Monad < MonadContract
      def valid?(val)
        return false unless (val.is_a? Mon::M::Monad)
        case val
        when Mon::M::List
          Mon::C::List.new(@nested_contract).valid?(val)
        when Mon::M::Maybe
          Mon::C::Maybe.new(@nested_contract).valid?(val)
        when Mon::M::Try
          Mon::C::Try.new(@nested_contract).valid?(val)
        when Mon::M::Lazy
          Mon::C::Lazy.new(@nested_contract).valid?(val)
        when Mon::M::Future
          Mon::C::Future.new(@nested_contract).valid?(val)
        when Mon::M::React
          Mon::C::React.new(@nested_contract).valid?(val)
        else
          raise RuntimeError.new("Unrecognized monad: #{ val.class }!")
        end
      end
    end
  end
end
