# Contract for the Mon::Try monad, for use with the Contracts gem
# For example:
# <tt>Contract C::Try[Num] => C::Try[Num]
# def trySquare(l)
#   l.bind { |i| i * i }
# end</tt>
# will work only for (eg) trySquare(Mon::Try.to { get_num_from_remote_service() }). If passed
# anything (or returning anything) other than a Mon::Try, a
# ContractViolation will be thrown.

module Mon
  module Contract
    require_relative 'monad_contract'
    
    class Try < MonadContract
      def valid?(val)
        # Should be a Try (i.e. success or failure), and if it's a 
        # success, the value should be valid re: the provided contract
        val.is_a?(Mon::M::Try) and (val.is_a?(Mon::M::Success).implies(valid_nested_contract?(val._)))
      end
    end
  end
end
