
module Mon

  module Monad

    require_relative 'chainable_monad'
    require_relative 'monad'

    class Try < Monad
      include ChainableMonad

      def self.[](obj)
        if obj.is_a? Failure
          Failure[obj.get]
        elsif obj
          Success[obj]
        else
          Failure[false]
        end
      end

      def self::to *obj, &f
        begin
          if obj.length == 1 and obj[0].is_a? Proc
            raise RuntimeError("Provided too many procs!") if f
            Success[obj.call]
          elsif f and obj.length > 0
            Success[f.call(*obj)]
          else
            raise RuntimeError("Failed to provide block") if not f
            Success[f.call]
          end
        rescue StandardError => e
          Failure[e]
        end
      end

      def self::valid?(o)
        o.is_a? self.class
      end

      class << self
        protected :new
      end
    end

    class Success < Try

      def initialize(obj)
        if obj.is_a? Success
          @obj = obj.get
        else
          @obj = obj
        end
        super()
      end
      
      def self.[](obj)
        if (obj.is_a? Failure) or (obj.is_a? Success)
          obj
        else
          Success.new(obj)
        end
      end

      def bind &fun
        begin
          Success[fun.call(@obj)]
        rescue StandardError => e
          Failure[e]
        end
      end

      def _canBind?(name)
        @obj.respond_to? name
      end

      def or obj, &f
        @obj
      end

      def orFail
        @obj
      end

      def unwrap
        @obj
      end
      
      def eql?(other)
        if other.is_a? Success
          @obj == other.unwrap
        else
          @obj == other
        end
      end

      def ==(o)
        eql?(o)
      end
      
      def equal?(o)
        eql?(o)
      end

      def to_s
        "Success[#{@obj}]"
      end

      def self::valid?(o)
        o.is_a? self.class
      end

      class << self
        protected :new
      end
      
      protected :unwrap, :_canBind?
    end

    class Failure < Try

      def initialize(err)
        @err = err
      end

      def self.[](err)
        self.new(err)
      end

      def bind &fun
        self
      end

      def or obj, &f
        if f and obj
          f.call(obj)
        elsif f
          f.call
        else
          obj
        end
      end

      def orFail &f
        if f
          f.call(@err)
        elsif @err.is_a? Exception
          raise @err
        else
          @err
        end
      end

      def unwrap
        @err
      end

      def _canBind? name
        true
      end

      def to_s
        "Failure[#{@err}]"
      end
      
      def eql?(other)
        (other.is_a? Failure) ? (@err == other.unwrap) : false
      end

      def ==(o)
        eql?(o)
      end
      
      def equal?(o)
        eql?(o)
      end

      def self::valid?(o)
        o.is_a? self.class
      end
      
      class << self
        protected :new
      end
      
      protected :unwrap, :_canBind?
    end
  end
end
